"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const GridRequestDto_1 = require("./GridRequestDto");
const GridResponse_1 = __importDefault(require("./GridResponse"));
class AGrid {
    constructor(db) {
        this.db = db;
        this.filterableColumns = null;
        this.sortableColumns = null;
        this.searchableColumns = null;
    }
    filter(dto) {
        return __awaiter(this, void 0, void 0, function* () {
            let query = this.searchQuery(this.db.query(this.entity)).filter(this.addFilter(dto));
            const count = yield query.count();
            query = this.addSorter(query, dto);
            query = this.addPaging(query, dto);
            const result = yield query.find();
            return new GridResponse_1.default(result, count, dto);
        });
    }
    searchQuery(query) {
        return query;
    }
    addPaging(query, dto) {
        const paging = dto.paging || {
            page: 1,
            itemsPerPage: 10,
        };
        paging.page = Math.max(paging.page, 1);
        paging.itemsPerPage = Math.max(paging.itemsPerPage, 1);
        dto.paging = paging;
        return query
            .skip((paging.page - 1) * paging.itemsPerPage)
            .limit(paging.itemsPerPage);
    }
    addSorter(query, dto) {
        if (!dto.sorter) {
            return query;
        }
        let sorted = query;
        dto.sorter.forEach(sorter => {
            let column = sorter.column;
            if (this.sortableColumns != null) {
                if (column in this.sortableColumns) {
                    column = this.sortableColumns[column];
                }
                else {
                    throw new Error(`[${column}] is not allowed for sorting`);
                }
            }
            sorted = sorted.orderBy(column, sorter.direction.toLowerCase());
        });
        return sorted;
    }
    addFilter(dto) {
        var _a;
        const filter = { $and: [] };
        if (dto.filter && dto.filter.length > 0) {
            dto.filter.forEach(and => {
                const orFilter = [];
                and.forEach(or => {
                    const column = this.getFilterableColumn(or.column);
                    orFilter.push({ [column]: AGrid.createExpression(or) });
                });
                filter.$and.push({ $or: orFilter });
            });
        }
        if (dto.search) {
            const searches = [];
            (_a = this.searchableColumns) === null || _a === void 0 ? void 0 : _a.forEach(column => {
                const name = this.getFilterableColumn(column);
                searches.push({ [name]: { $regex: dto.search } });
            });
            filter.$and.push({ $or: searches });
        }
        return filter;
    }
    getFilterableColumn(name) {
        let column = name;
        if (this.filterableColumns != null) {
            if (column in this.filterableColumns) {
                column = this.filterableColumns[column];
            }
            else {
                throw new Error(`[${column}] is not allowed for filtering`);
            }
        }
        return column;
    }
    static createExpression(filter) {
        switch (filter.operator) {
            case GridRequestDto_1.Operator.EQ:
                return filter.value[0];
            case GridRequestDto_1.Operator.NEQ:
                return { $ne: filter.value[0] };
            case GridRequestDto_1.Operator.IN:
                return { $in: filter.value };
            case GridRequestDto_1.Operator.NIN:
                return { $nin: filter.value };
            case GridRequestDto_1.Operator.GT:
                return { $gt: filter.value[0] };
            case GridRequestDto_1.Operator.LT:
                return { $lt: filter.value[0] };
            case GridRequestDto_1.Operator.GTE:
                return { $gte: filter.value[0] };
            case GridRequestDto_1.Operator.LTE:
                return { $lte: filter.value[0] };
            case GridRequestDto_1.Operator.LIKE:
                return { $regex: filter.value[0] };
            case GridRequestDto_1.Operator.STARTS:
                return { $regex: `^${filter.value[0]}` };
            case GridRequestDto_1.Operator.ENDS:
                return { $regex: `${filter.value[0]}$` };
            case GridRequestDto_1.Operator.NEMPTY:
                return { $ne: null };
            case GridRequestDto_1.Operator.EMPTY:
                return { $eq: null };
            case GridRequestDto_1.Operator.BETWEEN:
                return {
                    $gte: filter.value[0],
                    $lte: filter.value[1],
                };
            case GridRequestDto_1.Operator.NBETWEEN:
                return {
                    $gte: filter.value[1],
                    $lte: filter.value[0],
                };
        }
        return filter.value;
    }
}
exports.default = AGrid;
